<?php

namespace Drupal\search_api_lunr;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The PageAttachments class.
 */
class PageAttachments implements ContainerInjectionInterface {

  /**
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $serverStorage;

  /**
   * Create and instance of PageAttachments.
   */
  public function __construct($serverStorage) {
    $this->serverStorage = $serverStorage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')->getStorage('search_api_server')
    );
  }

  /**
   * Implements hook_page_attachments().
   */
  public function attachments(&$page) {
    $settings = [
      'servers' => [],
    ];
    /** @var \Drupal\search_api\ServerInterface[] $servers */
    /** @var \Drupal\search_api_lunr\Plugin\search_api\backend\SearchApiJsBackend $backend */
    $servers = $this->serverStorage->loadByProperties([
      'backend' => 'search_api_lunr',
    ]);
    foreach ($servers as $server) {
      foreach ($server->getIndexes() as $index) {
        $backend = $server->getBackend();
        $settings['servers'][$server->id()]['indexes'][$index->id()]['fileList'] = $backend->getIndexFileList($index);
        $settings['servers'][$server->id()]['indexes'][$index->id()]['fields'] = array_keys($index->getFields());
        $settings['servers'][$server->id()]['indexes'][$index->id()]['version'] = $backend->getIndexVersion($index);
      }
    }
    $page['#attached']['drupalSettings']['search_api_lunr'] = $settings;
    $page['#attached']['library'][] = 'search_api_lunr/api';
  }

}
